package com.steerinfo.dil.controller;

import com.github.pagehelper.PageHelper;
import com.steerinfo.dil.model.BmstruckInvoice;
import com.steerinfo.dil.service.impl.BmstruckInvoiceServiceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.FileNameUtils;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import oracle.jdbc.proxy.annotation.Post;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/24 17:29
 * @Version:V2.0
 */
@RestController
@RequestMapping("/${api.version}/bmstruckinvoice")
public class BmstruckInvoiceController extends BaseRESTfulController {

    @Autowired
    BmstruckInvoiceServiceImpl bmstruckInvoiceService;
    @Autowired
    ColumnDataUtil columnDataUtil;

    private final String UPLOAD_PATH = "C:/Download/Files/InvoicePhotos/";

    // 资源加载
    private final ResourceLoader resourceLoader;

    public BmstruckInvoiceController(ResourceLoader resourceLoader) {
        this.resourceLoader = resourceLoader;
    }

    /**
     * 新增发票
     *
     * @param mapVal
     * @return
     */
    @ApiModelProperty(value = "新增发票")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapVal", value = "销售账单id，托运方id，承运商id", required = false, dataType = "mapVal"),
    })
    @PostMapping("/addBmstruckInvoice")
    public RESTfulResult addBmstruckInvoice(@RequestBody Map<String, Object> mapVal) {
        int code = bmstruckInvoiceService.addBmstruckInvoice(mapVal);
        return success(code);
    }

    /**
     * 展示发票
     *
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示发票")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "178", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getInvoiceList")
    public RESTfulResult getInvoiceList(@RequestBody(required = false) Map<String, Object> mapValue,
                                        Integer pageNum,
                                        Integer pageSize,
                                        Integer apiId) {
        List<Map<String, Object>> list = bmstruckInvoiceService.getInvoiceList(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmstruckInvoiceService.getInvoiceList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 上传发票图片接口
     * @param invoiceId
     * @param file
     * @param request
     * @return
     */
    @ApiModelProperty(value = "上传发票图片接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "invoiceId", value = "发票id", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "file", value = "图片文件", required = false, dataType = "MultipartFile"),
            @ApiImplicitParam(name = "request", value = "请求", required = false, dataType = "HttpServletRequest")
    })
    @PostMapping("/singleFileUpdate/{invoiceId}")
    public RESTfulResult singleFileUpdate(@PathVariable("invoiceId") Integer invoiceId,
                                          @RequestParam MultipartFile file,
                                          HttpServletRequest request) {
        try {
            byte[] bytes = file.getBytes();
            String imageFileName = file.getOriginalFilename();
            String fileName = FileNameUtils.getPhotoName("imag", imageFileName);
            Path path = Paths.get(UPLOAD_PATH + fileName);
            BmstruckInvoice bmstruckInvoice = bmstruckInvoiceService.selectByPrimaryKey(new BigDecimal(invoiceId));
            bmstruckInvoice.setInvoiceImagePath(path.toString());
            bmstruckInvoice.setInvoiceBillingDate(new Date());
            int code = bmstruckInvoiceService.updateByPrimaryKeySelective(bmstruckInvoice);
            Files.write(path, bytes);
            return success(code);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return success();
    }

    /**
     * 查询发票图片
     *
     * @param response
     * @param invoiceId
     * @throws IOException
     */
    @ApiModelProperty(value = "查询发票图片")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "invoiceId", value = "发票id", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "response", value = "响应", required = false, dataType = "HttpServletResponse")
    })
    @GetMapping(value = "/getImage/{invoiceId}")
    public void getImage(HttpServletResponse response,
                         @PathVariable("invoiceId") Integer invoiceId) throws IOException {
        response.setContentType("image/jpeg;charset=utf-8");
        response.setHeader("Content-Disposition", "inline; filename=girls.png");
        ServletOutputStream outputStream = response.getOutputStream();
        BmstruckInvoice bmstruckInvoice = bmstruckInvoiceService.selectByPrimaryKey(new BigDecimal(invoiceId));
        String invoiceImagePath = bmstruckInvoice.getInvoiceImagePath();
        String replaceStr = invoiceImagePath.replace(UPLOAD_PATH, "换");
        int index = replaceStr.indexOf("换");
        String name = replaceStr.substring(index + 1);
        outputStream.write(Files.readAllBytes(Paths.get(UPLOAD_PATH).resolve(name)));
        outputStream.flush();
        outputStream.close();
    }

}
